/*
 * GTK+(2.0.x)ѤPC-UNIXѥĢΥ
 *
 * 󥿡ե뤿δؿ
 */

/*
 * Cmemo : Text Editor
 * interface.c - source file for  building interface
 *
 * Copyright (C) 2000, 2001, 2002 Seiji Tamura
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <pango/pango.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cme_def.h"
#include "cmemo.h"
#include "files.h"
#include "linenum.h"
#include "coloron.h"
#include "undo.h"
#include "warning.h"
#include "search.h"
#include "ini_file.h"
#include "interface.h"
#include "kanji_conv.h"
#include "cme_misc.h"
#include "bm_search.h"
#include "global_search.h"
#include "jump.h"
#include "auto_indent.h"
#include "keys.h"

#ifdef GTK20
#include "widget/gtkcmmenubar.h"
#undef gtk_menu_bar_new
#define gtk_menu_bar_new() gtk_cm_menu_bar_new()
#undef gtk_menu_bar_append
#define gtk_menu_bar_append(a, b) gtk_cm_menu_bar_append((a), (b))
#undef GTK_MENU_BAR
#define GTK_MENU_BAR GTK_CM_MENU_BAR
#endif /* #ifdef GTK20 */

struct combo_struct cm_dir;
struct combo_struct cm_ftype;
struct combo_struct cm_pat;
GtkWidget *cm_match_chk_btn;
GtkWidget *cm_regex_chk_btn;
GdkColor line_fg_color;
GdkColor ctrl_color;
static GtkWidget *toolbar_visible_item;
static GtkWidget *line_wrap_item;
static GtkWidget *line_numbers_item;
static GtkWidget *syntax_highlighting_item;

/* local declarations */
static int create_lang_menu(GtkWidget *lang_menu, GtkAccelGroup *accel_group);
static int create_mode_menu(GtkWidget *mode_menu);


/********************
   Lang˥塼
 ********************/
static int create_lang_menu(GtkWidget *lang_menu, GtkAccelGroup *accel_group)
{
  int i;
  GtkWidget *sub_item;

  /* ftype_info¤Τξ˽äLang˥塼 */
  for (i = 0; i < FTYPE_INFO_MAX; i++) {
    cm_ft_inf[i].mode_id = i;
    if (cm_ft_inf[i].data_flag == 0) {
      continue;
    }
    sub_item = gtk_menu_item_new_with_label(cm_ft_inf[i].title);
    gtk_menu_append(GTK_MENU(lang_menu), sub_item);
    gtk_widget_add_accelerator(sub_item, "activate",
                               accel_group, cm_ft_inf[i].acckey,
                               GDK_SHIFT_MASK | GDK_MOD1_MASK,
                               GTK_ACCEL_VISIBLE);
    gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                       GTK_SIGNAL_FUNC(mode_change_cb),
                       &cm_ft_inf[i]);
    if (cm_ft_inf[i].enabled == 0) {
      gtk_widget_set_sensitive(sub_item, FALSE);
    }
    gtk_widget_show(sub_item);
  }
  return(0);
}

/********************
   Mode˥塼
 ********************/
static int create_mode_menu(GtkWidget *mode_menu)
{
  int i;
  GtkWidget *sub_item;

  /*(ʸɥ⡼)*/
  /* kmode_info¤Τξ˽äLang˥塼 */
  for (i = 0; i < KMODE_INFO_MAX; i++) {
    /*cm_km_inf[i].mode_id = i;*/
    if (cm_km_inf[i].data_flag == 0) {
      continue;
    }
    sub_item = gtk_menu_item_new_with_label(cm_km_inf[i].title);
    gtk_menu_append(GTK_MENU(mode_menu), sub_item);
    gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                       GTK_SIGNAL_FUNC(set_kanji_mode),
                       &cm_km_inf[i]);
    if (cm_km_inf[i].enabled == 0) {
      gtk_widget_set_sensitive(sub_item, FALSE);
    }
    gtk_widget_show(sub_item);
  }
  return(0);
}

/************************************
   ˥塼С˥˥塼
 ************************************/
GtkWidget *create_menu(GtkAccelGroup *accel_group)
{
  GtkWidget *menu_bar;
  GtkWidget *sub_item;
  GtkWidget *file_menu;
  GtkWidget *file_item;
  GtkWidget *edit_menu;
  GtkWidget *edit_item;
  GtkWidget *search_menu;
  GtkWidget *search_item;
  GtkWidget *view_menu;
  GtkWidget *view_item;
  GtkWidget *cmds_menu;
  GtkWidget *cmds_item;
  GtkWidget *lang_menu;
  GtkWidget *lang_item;
  GtkWidget *mode_menu;
  GtkWidget *mode_item;
  GtkWidget *help_menu;
  GtkWidget *help_item;
  GtkWidget *hsep;


  menu_bar = gtk_menu_bar_new();
  gtk_widget_show(menu_bar);
//  gtk_container_set_border_width (GTK_CONTAINER(menu_bar), 0);

  /********************************
     ˥塼 "File" (ե)
   ********************************/
  file_menu = gtk_menu_new();

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_new_window);
  gtk_menu_append(GTK_MENU(file_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'N', GDK_MOD1_MASK, 1);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                     GTK_SIGNAL_FUNC(new_window), NULL);
  gtk_widget_show(sub_item);

  hsep = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(file_menu), hsep);
  gtk_widget_set_sensitive(hsep, FALSE);
  gtk_widget_show(hsep);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_open);
  gtk_menu_append(GTK_MENU(file_menu), sub_item);
  /*gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'O', GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);*/
  /*gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'I', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);*/
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'O', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                     GTK_SIGNAL_FUNC(open_text),
                     (gpointer)&cm_document);
  gtk_widget_show(sub_item);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_save);
  gtk_menu_append(GTK_MENU(file_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'S', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(save_text),(gpointer)&cm_document);
  gtk_widget_show(sub_item);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_save_as);
  gtk_menu_append(GTK_MENU(file_menu), sub_item);
  /*gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'S', GDK_MOD1_MASK, 1);*/
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'W', GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(save_as_text),(gpointer)&cm_document);
  gtk_widget_show(sub_item);

  hsep = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(file_menu), hsep);
  gtk_widget_set_sensitive(hsep, FALSE);
  gtk_widget_show(hsep);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_print);
  gtk_menu_append(GTK_MENU(file_menu), sub_item);
  /*gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'R', GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);*/
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(print_cb), NULL);
  gtk_widget_set_sensitive(sub_item, FALSE);
  gtk_widget_show(sub_item);

  hsep = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(file_menu), hsep);
  gtk_widget_set_sensitive(hsep, FALSE);
  gtk_widget_show(hsep);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_exit);
  gtk_menu_append(GTK_MENU(file_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'Q', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(main_win_destroy), (gpointer)cm_window);
  gtk_widget_show(sub_item);

  file_item = gtk_menu_item_new_with_mnemonic(cm_msg_menu_file);
  gtk_widget_show(file_item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_item), file_menu);

  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), file_item);


  /****************************
     ˥塼 "Edit" (Խ)
   ****************************/
  edit_menu = gtk_menu_new();

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_undo);
  gtk_menu_append(GTK_MENU(edit_menu), sub_item);
  /*gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             '/', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);*/
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'Z', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item),"activate",
		     GTK_SIGNAL_FUNC(undo_cb), (gpointer)cm_document.textbox);
  gtk_widget_show(sub_item);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_redo);
  gtk_menu_append(GTK_MENU(edit_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'Z', GDK_CONTROL_MASK | GDK_SHIFT_MASK,
                             GTK_ACCEL_VISIBLE);
//  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
//                             'R', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item),"activate",
		     GTK_SIGNAL_FUNC(redo_cb), (gpointer)cm_document.textbox);
  gtk_widget_show(sub_item);

  hsep = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(edit_menu), hsep);
  gtk_widget_set_sensitive(hsep, FALSE);
  gtk_widget_show(hsep);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_copy);
  gtk_menu_append(GTK_MENU(edit_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'C', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(edit_copy), NULL);
  gtk_widget_show(sub_item);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_cut);
  gtk_menu_append(GTK_MENU(edit_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'X', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(edit_cut), NULL);
  gtk_widget_show(sub_item);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_paste);
  gtk_menu_append(GTK_MENU(edit_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'V', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(edit_paste), NULL);
  gtk_widget_show(sub_item);

  hsep = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(edit_menu), hsep);
  gtk_widget_set_sensitive(hsep, FALSE);
  gtk_widget_show(hsep);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_select_all);
  gtk_menu_append(GTK_MENU(edit_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'A', GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
       GTK_SIGNAL_FUNC(select_all_cb), NULL);
  gtk_widget_show(sub_item);

  edit_item = gtk_menu_item_new_with_mnemonic(cm_msg_menu_edit);
  gtk_widget_show(edit_item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(edit_item), edit_menu);

  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), edit_item);


  /******************************
     ˥塼 "Search" ()
   ******************************/
  search_menu = gtk_menu_new();

  /* ... */
  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_find);
  gtk_menu_append(GTK_MENU(search_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
//                             'X', GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
                             'G', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                     GTK_SIGNAL_FUNC(search_visi_cb), NULL);
  gtk_widget_show(sub_item);
/*
  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_find_dlg_open);
  gtk_menu_append(GTK_MENU(search_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'G', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
       GTK_SIGNAL_FUNC(search_open_cb), NULL);
  gtk_widget_show(sub_item);
*/
  /*  */
  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_search_dn);
  gtk_menu_append(GTK_MENU(search_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'Y', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             GDK_F3, 0, 0);
  gtk_signal_connect(GTK_OBJECT(sub_item),
                     "activate",
                     GTK_SIGNAL_FUNC(dn_search),
                     &cm_document);
  gtk_widget_show(sub_item);

  /* 帡 */
  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_search_up);
  gtk_menu_append(GTK_MENU(search_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'T', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             GDK_F3, GDK_SHIFT_MASK, 0);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             GDK_F4, 0, 0);
  gtk_signal_connect(GTK_OBJECT(sub_item),
                     "activate",
                     GTK_SIGNAL_FUNC(up_search),
                     &cm_document);
  gtk_widget_show(sub_item);

  /* ִ... */
  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_replace);
  gtk_menu_append(GTK_MENU(search_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'R', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                     GTK_SIGNAL_FUNC(replace_visi_cb), NULL);
  gtk_widget_show(sub_item);

  /* ʸʸ̵롩 */
  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_ignore_case);
  gtk_menu_append(GTK_MENU(search_menu), sub_item);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                     GTK_SIGNAL_FUNC(case_on_off), NULL);
  gtk_widget_show(sub_item);

  /* ɽѤ롩 */
  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_regex_search);
  gtk_menu_append(GTK_MENU(search_menu), sub_item);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                     GTK_SIGNAL_FUNC(case_on_off), NULL);
  gtk_widget_set_sensitive(sub_item, FALSE);
  gtk_widget_show(sub_item);

  /* ѥ졼 */
  hsep = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(search_menu), hsep);
  gtk_widget_set_sensitive(hsep, FALSE);
  gtk_widget_show(hsep);

  /* Ԥإ */
  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_jump_to);
  gtk_menu_append(GTK_MENU(search_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'J', GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
       GTK_SIGNAL_FUNC(jump_visi_cb), NULL);
  gtk_widget_show(sub_item);

  /* ѥ졼 */
  hsep = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(search_menu), hsep);
  gtk_widget_set_sensitive(hsep, FALSE);
  gtk_widget_show(hsep);

  /*  */
  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_tag_jump);
  gtk_menu_append(GTK_MENU(search_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'K', GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             GDK_F12, 0, 0);
  gtk_signal_connect(GTK_OBJECT(sub_item),
                     "activate",
                     GTK_SIGNAL_FUNC(tag_jump_cb),
                     &cm_document);
  gtk_widget_show(sub_item);

  /* Х븡 */
  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_global_search);
  gtk_menu_append(GTK_MENU(search_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'G', GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item),
                     "activate",
                     GTK_SIGNAL_FUNC(gsearch_run_cb),
                     &cm_document);
  gtk_widget_show(sub_item);

  /* ˥塼ָ */
  search_item = gtk_menu_item_new_with_mnemonic(cm_msg_menu_search);
  gtk_widget_show(search_item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(search_item), search_menu);

  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), search_item);


  /*******************
     ˥塼 "View"
   *******************/
  view_menu = gtk_menu_new();

  /*sub_item = gtk_menu_item_new_with_label("color on");
  gtk_menu_append(GTK_MENU(view_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'L', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(color_on_cb), NULL);
  gtk_widget_show(sub_item);*/

  /*sub_item = gtk_menu_item_new_with_label("color off");
  gtk_menu_append(GTK_MENU(view_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'L', GDK_CONTROL_MASK | GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(color_off_cb), NULL);
  gtk_widget_show(sub_item);

  hsep = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(view_menu), hsep);
  gtk_widget_set_sensitive(hsep, FALSE);
  gtk_widget_show(hsep);*/

  sub_item = gtk_check_menu_item_new_with_label(cm_msg_menu_toolbar_visible);
  toolbar_visible_item = sub_item;
  gtk_menu_append(GTK_MENU(view_menu), sub_item);
  /*gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'P', GDK_SHIFT_MASK | GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);*/
  gtk_signal_connect(GTK_OBJECT(sub_item),
                     "activate",
                     GTK_SIGNAL_FUNC(toolbar_visible_cb),
                     &cm_document);
  gtk_widget_show(sub_item);

  sub_item = gtk_check_menu_item_new_with_label(cm_msg_menu_line_wrap);
  line_wrap_item = sub_item;
  gtk_menu_append(GTK_MENU(view_menu), sub_item);
  /*gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'P', GDK_SHIFT_MASK | GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);*/
  gtk_signal_connect(GTK_OBJECT(sub_item),
                     "activate",
                     GTK_SIGNAL_FUNC(line_wrap_cb),
                     &cm_document);
  gtk_widget_show(sub_item);

  sub_item = gtk_check_menu_item_new_with_label(cm_msg_menu_line_numbers);
  line_numbers_item = sub_item;
  gtk_menu_append(GTK_MENU(view_menu), sub_item);
  /*gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'P', GDK_SHIFT_MASK | GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);*/
  gtk_signal_connect(GTK_OBJECT(sub_item),
                     "activate",
                     GTK_SIGNAL_FUNC(show_line_numbers_cb),
                     &cm_document);
  gtk_widget_show(sub_item);

  sub_item = gtk_check_menu_item_new_with_label(cm_msg_menu_syntax_highlighting);
  syntax_highlighting_item = sub_item;
  gtk_menu_append(GTK_MENU(view_menu), sub_item);
  /*gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'L', GDK_CONTROL_MASK | GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);*/
  gtk_signal_connect(GTK_OBJECT(sub_item),
                     "activate",
                     GTK_SIGNAL_FUNC(syntax_highlighting_cb),
                     NULL);
  gtk_widget_show(sub_item);

  hsep = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(view_menu), hsep);
  gtk_widget_set_sensitive(hsep, FALSE);
  gtk_widget_show(hsep);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_refresh_display);
  gtk_menu_append(GTK_MENU(view_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'L', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(refresh_display_cb), &cm_document);
  gtk_widget_show(sub_item);

  view_item = gtk_menu_item_new_with_mnemonic(cm_msg_menu_view);
  gtk_widget_show(view_item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(view_item), view_menu);

  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), view_item);


  /******************************
     ˥塼 "Cmds" (Commands)
   ******************************/
  cmds_menu = gtk_menu_new();

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_play_key_macro);
  gtk_menu_append(GTK_MENU(cmds_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'P', GDK_SHIFT_MASK | GDK_CONTROL_MASK,
                             GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(key_macro_play_cb), NULL);
  gtk_widget_show(sub_item);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_record_key_macro);
  gtk_menu_append(GTK_MENU(cmds_menu), sub_item);
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'R', GDK_SHIFT_MASK | GDK_CONTROL_MASK,
                             GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(key_macro_record_cb), NULL);
  gtk_widget_show(sub_item);

  cmds_item = gtk_menu_item_new_with_mnemonic(cm_msg_menu_cmds);
  gtk_widget_show(cmds_item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(cmds_item), cmds_menu);

  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), cmds_item);


  /*******************
     ˥塼 "Lang"
   *******************/
  /*()*/
  lang_menu = gtk_menu_new();

  /*ftype_info¤ΤΥǡ˽äƥ˥塼 */
  create_lang_menu(lang_menu, accel_group);

  /*(ե륿)*/
  lang_item = gtk_menu_item_new_with_mnemonic(cm_msg_menu_lang);
  gtk_widget_show(lang_item);

  /*˥塼"Lang"˥å*/
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(lang_item), lang_menu);

  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), lang_item);


  /*******************
     ˥塼 "Mode"
   *******************/
  /*(Ƽ⡼)*/
  mode_menu = gtk_menu_new();

  /*kmode_info¤ΤΥǡ˽äƥ˥塼 */
  create_mode_menu(mode_menu);

  /*ѥ졼*/
  hsep = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(mode_menu), hsep);
  gtk_widget_set_sensitive(hsep, FALSE);
  gtk_widget_show(hsep);

  /*(ʸΥ⡼)*/
  sub_item = gtk_menu_item_new_with_label(cm_msg_return_code_cr);
  gtk_menu_append(GTK_MENU(mode_menu), sub_item);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                     GTK_SIGNAL_FUNC(set_return_mode),
                     (gpointer)CME_CR);
  gtk_widget_show(sub_item);

  sub_item = gtk_menu_item_new_with_label(cm_msg_return_code_lf);
  gtk_menu_append(GTK_MENU(mode_menu), sub_item);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                     GTK_SIGNAL_FUNC(set_return_mode),
                     (gpointer)CME_LF);
  gtk_widget_show(sub_item);

  sub_item = gtk_menu_item_new_with_label(cm_msg_return_code_crlf);
  gtk_menu_append(GTK_MENU(mode_menu), sub_item);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                     GTK_SIGNAL_FUNC(set_return_mode),
                     (gpointer)CME_CRLF);
  gtk_widget_show(sub_item);

  /*ѥ졼*/
  hsep = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(mode_menu), hsep);
  gtk_widget_set_sensitive(hsep, FALSE);
  gtk_widget_show(hsep);

  /*⡼ɤߥ˥Хåեɽ*/
  sub_item = gtk_menu_item_new_with_label(cm_msg_show_mode);
  gtk_menu_append(GTK_MENU(mode_menu), sub_item);
  /*gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'M', GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);*/
  gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             'M', GDK_CONTROL_MASK | GDK_MOD1_MASK,
                             GTK_ACCEL_VISIBLE);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
                     GTK_SIGNAL_FUNC(show_mode_cb), NULL);
  gtk_widget_show(sub_item);

  /*⡼*/
  mode_item = gtk_menu_item_new_with_mnemonic(cm_msg_mode);
  gtk_widget_show(mode_item);

  /*⡼ɥ˥塼"Mode"˥å*/
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(mode_item), mode_menu);

  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), mode_item);


  /******************************
     ˥塼 "Help" ()
   ******************************/
  help_menu = gtk_menu_new();

  sub_item = gtk_menu_item_new_with_label(cm_msg_settings);
  gtk_menu_append(GTK_MENU(help_menu), sub_item);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(show_ini), NULL);
  gtk_widget_show(sub_item);

  sub_item = gtk_menu_item_new_with_label(cm_msg_version_info);
  gtk_menu_append(GTK_MENU(help_menu), sub_item);
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(version_cb), NULL);
  gtk_widget_show(sub_item);

  help_item = gtk_menu_item_new_with_mnemonic(cm_msg_help);
  gtk_widget_show(help_item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(help_item), help_menu);

  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), help_item);

  gtk_menu_item_right_justify(GTK_MENU_ITEM(help_item));

  /* ˥塼Сι⤵ */
#ifdef GTK20
//  gtk_widget_set_size_request(menu_bar, -1, MENUBAR_HEIGHT);
#endif
  /* Ǹ˥˥塼СαƤ̵(եå)ꤹ */
  /*gtk_menu_bar_set_shadow_type (GTK_MENU_BAR(menu_bar), GTK_SHADOW_NONE);*/

  return(menu_bar);
}

/************************************************
   ˥塼ȥġСζοʿ
 ************************************************/
GtkWidget *create_hseparator()
{
  return(gtk_hseparator_new());
}

/************************
   ġС
 ************************/
GtkWidget *create_toolbar()
{
  GtkWidget *toolbar;
  GtkWidget *tool_button;
//  GtkToolItem* tool_item;
//  GtkWidget *iconw;

  toolbar = gtk_toolbar_new();
  gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar), GTK_ORIENTATION_HORIZONTAL);
//  gtk_toolbar_set_style (GTK_TOOLBAR(toolbar), GTK_TOOLBAR_BOTH);
  gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
  gtk_container_set_border_width(GTK_CONTAINER(toolbar), 0);

//  tool_item = gtk_tool_button_new_from_stock(GTK_STOCK_NEW);
//  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tool_item, -1);

  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_NEW,
                                         cm_msg_menu_new_window,
                                         "New Window",
                                         GTK_SIGNAL_FUNC(new_window),
                                         NULL,
                                         -1);
  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_OPEN,
                                         cm_msg_menu_open,
                                         "Open",
                                         GTK_SIGNAL_FUNC(open_text),
                                         (gpointer)&cm_document,
                                         -1);
  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_SAVE,
                                         cm_msg_menu_save,
                                         "Save",
                                         GTK_SIGNAL_FUNC(save_text),
                                         (gpointer)&cm_document,
                                         -1);
  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_SAVE_AS,
                                         cm_msg_menu_save_as,
                                         "Save as",
                                         GTK_SIGNAL_FUNC(save_as_text),
                                         (gpointer)&cm_document,
                                         -1);
  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_UNDO,
                                         cm_msg_menu_undo,
                                         "Undo",
                                         GTK_SIGNAL_FUNC(undo_cb),
                                         (gpointer)cm_document.textbox,
                                         -1);
  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_REDO,
                                         cm_msg_menu_redo,
                                         "Redo",
                                         GTK_SIGNAL_FUNC(redo_cb),
                                         (gpointer)cm_document.textbox,
                                         -1);
  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_COPY,
                                         cm_msg_menu_copy,
                                         "Copy",
                                         GTK_SIGNAL_FUNC(edit_copy),
                                         NULL,
                                         -1);
  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_CUT,
                                         cm_msg_menu_cut,
                                         "Cut",
                                         GTK_SIGNAL_FUNC(edit_cut),
                                         NULL,
                                         -1);
  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_PASTE,
                                         cm_msg_menu_paste,
                                         "Paste",
                                         GTK_SIGNAL_FUNC(edit_paste),
                                         NULL,
                                         -1);
  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_GO_DOWN,
                                         cm_msg_menu_search_dn,
                                         "Search Dn",
                                         GTK_SIGNAL_FUNC(dn_search),
                                         &cm_document,
                                         -1);
  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_GO_UP,
                                         cm_msg_menu_search_up,
                                         "Search Up",
                                         GTK_SIGNAL_FUNC(up_search),
                                         &cm_document,
                                         -1);
  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_FIND,
                                         cm_msg_menu_find,
                                         "Find...",
                                         GTK_SIGNAL_FUNC(search_visi_cb),
                                         NULL,
                                         -1);
  tool_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
                                         GTK_STOCK_FIND_AND_REPLACE,
                                         cm_msg_menu_replace,
                                         "Replace...",
                                         GTK_SIGNAL_FUNC(replace_visi_cb),
                                         NULL,
                                         -1);

  gtk_toolbar_set_icon_size(toolbar, GTK_ICON_SIZE_MENU);
  return(toolbar);
}

/**********************************
   ݥåץåץ˥塼
 **********************************/
GtkWidget *create_popup_menu(GtkWidget *widget,
                             int text_entry,
                             GtkAccelGroup *accel_group)
{
  GtkWidget *popup_menu;
  GtkWidget *sub_item;
  GtkWidget *hsep;

  /*ݥåץåץ˥塼*/
  popup_menu = gtk_menu_new();

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_copy);
  gtk_menu_append(GTK_MENU(popup_menu), sub_item);
  if (text_entry == 1) {
    gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                             GDK_F6, 0, 1);
  }
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(edit_copy), (gpointer)widget);
  gtk_widget_show(sub_item);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_cut);
  gtk_menu_append(GTK_MENU(popup_menu), sub_item);
  if (text_entry == 1) {
    gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                               GDK_F7, 0, 1);
  }
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(edit_cut), (gpointer)widget);
  gtk_widget_show(sub_item);

  sub_item = gtk_menu_item_new_with_label(cm_msg_menu_paste);
  gtk_menu_append(GTK_MENU(popup_menu), sub_item);
  if (text_entry == 1) {
    gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                               GDK_F8, 0, 1);
  }
  gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
		     GTK_SIGNAL_FUNC(edit_paste), (gpointer)widget);
  gtk_widget_show(sub_item);

  if (text_entry == 1) {
    hsep = gtk_menu_item_new();
    gtk_menu_append(GTK_MENU(popup_menu), hsep);
    gtk_widget_set_sensitive(hsep, FALSE);
    gtk_widget_show(hsep);

    sub_item = gtk_menu_item_new_with_label(cm_msg_menu_select_all);
    gtk_menu_append(GTK_MENU(popup_menu), sub_item);
    gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                               GDK_F9, 0, 1);
    gtk_signal_connect(GTK_OBJECT(sub_item), "activate",
               GTK_SIGNAL_FUNC(select_all_cb), NULL);
    gtk_widget_show(sub_item);

    hsep = gtk_menu_item_new();
    gtk_menu_append(GTK_MENU(popup_menu), hsep);
    gtk_widget_set_sensitive(hsep, FALSE);
    gtk_widget_show(hsep);

    sub_item = gtk_menu_item_new_with_label(cm_msg_menu_tag_jump);
    gtk_menu_append(GTK_MENU(popup_menu), sub_item);
    gtk_widget_add_accelerator(sub_item, "activate", accel_group,
                               GDK_F12, 0, 1);
    gtk_signal_connect(GTK_OBJECT(sub_item),
                       "activate",
                       GTK_SIGNAL_FUNC(tag_jump_cb),
                       &cm_document);
    gtk_widget_show(sub_item);
  }

  return popup_menu;
}

/******************************************
   ưɽΥåȤ
 ******************************************/
int create_hidden_widget(GtkWidget *bottom_hbox, struct search_struct *srch_obj)
{
  /*帡ѥܥ*/
  srch_obj->button_up = gtk_button_new_with_label("up");
  gtk_button_set_focus_on_click(GTK_BUTTON(srch_obj->button_up), FALSE);
//  gtk_box_pack_start(GTK_BOX(bottom_hbox), srch_obj->button_up, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(srch_obj->button_up),
                     "clicked",
                     GTK_SIGNAL_FUNC(up_search),
                     &cm_document);
  gtk_button_set_relief(GTK_BUTTON(srch_obj->button_up),GTK_RELIEF_NONE);

  /*ѥܥ*/
  srch_obj->button_dn = gtk_button_new_with_label("dn");
  gtk_button_set_focus_on_click(GTK_BUTTON(srch_obj->button_dn), FALSE);
//  gtk_box_pack_start(GTK_BOX(bottom_hbox), srch_obj->button_dn, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(srch_obj->button_dn),
                     "clicked",
                     GTK_SIGNAL_FUNC(dn_search),
                     &cm_document);
  gtk_button_set_relief(GTK_BUTTON(srch_obj->button_dn),GTK_RELIEF_NONE);

  /*ʸϤEntryWidgetκ*/
  srch_obj->entry = gtk_entry_new();
  gtk_widget_set_size_request(srch_obj->entry, 130, -1);
//  gtk_box_pack_start(GTK_BOX(bottom_hbox), srch_obj->entry, FALSE, FALSE, 0);
  gtk_entry_set_editable(GTK_ENTRY(srch_obj->entry), TRUE);

  /*ִʸϤEntryWidgetκ*/
  srch_obj->rep_entry = gtk_entry_new();
  gtk_widget_set_size_request(srch_obj->rep_entry, 130, -1);
//  gtk_box_pack_start(GTK_BOX(bottom_hbox), srch_obj->rep_entry, FALSE, FALSE, 0);
  gtk_entry_set_editable(GTK_ENTRY(srch_obj->rep_entry), TRUE);
#ifdef GTK20
  gtk_widget_set_size_request(srch_obj->entry, 130, ENTRY_HEIGHT);
  gtk_widget_set_size_request(srch_obj->rep_entry, 130, ENTRY_HEIGHT);
  gtk_widget_set_size_request(srch_obj->button_up, -1, ENTRY_HEIGHT);
  gtk_widget_set_size_request(srch_obj->button_dn, -1, ENTRY_HEIGHT);
#endif
  gtk_box_pack_start(GTK_BOX(bottom_hbox), srch_obj->entry, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(bottom_hbox), srch_obj->rep_entry, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(bottom_hbox), srch_obj->button_up, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(bottom_hbox), srch_obj->button_dn, FALSE, FALSE, 0);

  /*ԥѥܥ*/
  cm_jump.button = gtk_button_new_with_label("jump");
//  gtk_box_pack_start(GTK_BOX(bottom_hbox), cm_jump.button, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(cm_jump.button),
                     "clicked",
                     GTK_SIGNAL_FUNC(line_jump_cb),
                     &cm_document);
  gtk_button_set_relief(GTK_BUTTON(cm_jump.button),GTK_RELIEF_NONE);

  /*פֹϤEntryWidgetκ*/
  cm_jump.entry = gtk_entry_new();
  gtk_widget_set_size_request(cm_jump.entry, 70, -1);
//  gtk_box_pack_start(GTK_BOX(bottom_hbox), cm_jump.entry, FALSE, FALSE, 0);
  gtk_entry_set_editable(GTK_ENTRY(cm_jump.entry), TRUE);
#ifdef GTK20
  gtk_widget_set_size_request(cm_jump.entry, 70, ENTRY_HEIGHT);
  gtk_widget_set_size_request(cm_jump.button, -1, ENTRY_HEIGHT);
#endif
  gtk_box_pack_start(GTK_BOX(bottom_hbox), cm_jump.entry, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(bottom_hbox), cm_jump.button, FALSE, FALSE, 0);
  return(0);
}

/************************************************
   Х륵ѥɥκ
 ************************************************/
/* δؿ Glade ΤԽƺ */
GtkWidget *create_gsearch_wnd(void)
{
  GtkWidget *gsearch_wnd;
  GtkWidget *vbox;
  GtkWidget *dummy_hbox;
  GtkWidget *dummy_label;
  GtkWidget *dir_hbox;
  GtkWidget *dir_label;
  GtkWidget *sel_btn;
  GtkWidget *ftype_hbox;
  GtkWidget *ftype_label;
  GtkWidget *pat_hbox;
  GtkWidget *pat_label;
  GtkWidget *opt_hbox;
  GtkWidget *ok_btn;
  GtkWidget *cancel_btn;
  PangoFontDescription *font_desc;
  char text_font[1024];

  /* ե̾ܥ */
  sprintf(text_font, "%s %d", cm_text_font, cm_font_size);

  gsearch_wnd = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (gsearch_wnd), "gsearch_wnd", gsearch_wnd);
  gtk_widget_set_size_request (gsearch_wnd, 400, 104);
  gtk_window_set_title (GTK_WINDOW (gsearch_wnd), "global search");
  gtk_window_set_position (GTK_WINDOW (gsearch_wnd), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (gsearch_wnd), TRUE);
  gtk_window_set_policy (GTK_WINDOW (gsearch_wnd), FALSE, FALSE, FALSE);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "vbox", vbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (gsearch_wnd), vbox);

  dummy_hbox = gtk_hbox_new (TRUE, 0);
  gtk_widget_ref (dummy_hbox);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "dummy_hbox", dummy_hbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dummy_hbox);
  gtk_box_pack_start (GTK_BOX (vbox), dummy_hbox, TRUE, TRUE, 0);
  gtk_widget_set_size_request (dummy_hbox, -1, 3);

  dummy_label = gtk_label_new ("");
  gtk_widget_ref (dummy_label);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "dummy_label", dummy_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dummy_label);
  gtk_box_pack_start (GTK_BOX (dummy_hbox), dummy_label, TRUE, TRUE, 0);

  dir_hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (dir_hbox);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "dir_hbox", dir_hbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dir_hbox);
  gtk_box_pack_start (GTK_BOX (vbox), dir_hbox, TRUE, FALSE, 0);

  dir_label = gtk_label_new (" directory ");
  gtk_widget_ref (dir_label);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "dir_label", dir_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dir_label);
  gtk_box_pack_start (GTK_BOX (dir_hbox), dir_label, FALSE, FALSE, 0);

  cm_dir.combo = gtk_combo_new ();
  gtk_widget_ref (cm_dir.combo);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "dir_combo", cm_dir.combo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cm_dir.combo);
  gtk_box_pack_start (GTK_BOX (dir_hbox), cm_dir.combo, TRUE, TRUE, 0);

  cm_dir.entry = GTK_COMBO (cm_dir.combo)->entry;
  gtk_widget_ref (cm_dir.entry);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "dir_entry", cm_dir.entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cm_dir.entry);

  sel_btn = gtk_button_new_with_label (" Select ");
  gtk_widget_ref (sel_btn);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "sel_btn", sel_btn,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (sel_btn);
  gtk_box_pack_start (GTK_BOX (dir_hbox), sel_btn, FALSE, TRUE, 0);

  gtk_signal_connect (GTK_OBJECT (sel_btn), "clicked",
                      GTK_SIGNAL_FUNC (sel_btn_cb),
                      (gpointer)gsearch_wnd);

  ftype_hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (ftype_hbox);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "ftype_hbox", ftype_hbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ftype_hbox);
  gtk_box_pack_start (GTK_BOX (vbox), ftype_hbox, TRUE, FALSE, 0);

  ftype_label = gtk_label_new (" filetype  ");
  gtk_widget_ref (ftype_label);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "ftype_label", ftype_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ftype_label);
  gtk_box_pack_start (GTK_BOX (ftype_hbox), ftype_label, FALSE, FALSE, 0);

  cm_ftype.combo = gtk_combo_new ();
  gtk_widget_ref (cm_ftype.combo);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "ftype_combo", cm_ftype.combo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cm_ftype.combo);
  gtk_box_pack_start (GTK_BOX (ftype_hbox), cm_ftype.combo, TRUE, TRUE, 0);

  cm_ftype.entry = GTK_COMBO (cm_ftype.combo)->entry;
  gtk_widget_ref (cm_ftype.entry);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "ftype_entry", cm_ftype.entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cm_ftype.entry);

  pat_hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (pat_hbox);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "pat_hbox", pat_hbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pat_hbox);
  gtk_box_pack_start (GTK_BOX (vbox), pat_hbox, TRUE, FALSE, 0);

  pat_label = gtk_label_new (" pattern   ");
  gtk_widget_ref (pat_label);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "pat_label", pat_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pat_label);
  gtk_box_pack_start (GTK_BOX (pat_hbox), pat_label, FALSE, FALSE, 0);

  cm_pat.combo = gtk_combo_new ();
  gtk_widget_ref (cm_pat.combo);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "pat_combo", cm_pat.combo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cm_pat.combo);
  gtk_box_pack_start (GTK_BOX (pat_hbox), cm_pat.combo, TRUE, TRUE, 0);

  cm_pat.entry = GTK_COMBO (cm_pat.combo)->entry;
  gtk_widget_ref (cm_pat.entry);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "pat_entry", cm_pat.entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cm_pat.entry);

  opt_hbox = gtk_hbox_new (TRUE, 0);
  gtk_widget_ref (opt_hbox);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "opt_hbox", opt_hbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (opt_hbox);
  gtk_box_pack_start (GTK_BOX (vbox), opt_hbox, TRUE, TRUE, 0);
//  gtk_widget_set_size_request (opt_hbox, -1, 25);
  gtk_container_set_border_width (GTK_CONTAINER (opt_hbox), 2);

  cm_match_chk_btn = gtk_check_button_new_with_label ("match case");
  gtk_widget_ref (cm_match_chk_btn);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "match_chk_btn",
                            cm_match_chk_btn, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cm_match_chk_btn);
  gtk_box_pack_start (GTK_BOX (opt_hbox), cm_match_chk_btn, FALSE, FALSE, 0);

  cm_regex_chk_btn = gtk_check_button_new_with_label ("regex");
  gtk_widget_ref (cm_regex_chk_btn);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "regex_chk_btn",
                            cm_regex_chk_btn, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cm_regex_chk_btn);
  gtk_box_pack_start (GTK_BOX (opt_hbox), cm_regex_chk_btn, FALSE, FALSE, 0);

  ok_btn = gtk_button_new_with_label ("OK");
  gtk_widget_ref (ok_btn);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "ok_btn", ok_btn,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ok_btn);
  gtk_box_pack_start (GTK_BOX (opt_hbox), ok_btn, FALSE, TRUE, 0);

  cancel_btn = gtk_button_new_with_label ("Cancel");
  gtk_widget_ref (cancel_btn);
  gtk_object_set_data_full (GTK_OBJECT (gsearch_wnd), "cancel_btn", cancel_btn,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cancel_btn);
  gtk_box_pack_start (GTK_BOX (opt_hbox), cancel_btn, FALSE, TRUE, 0);

  /* Хåؿ³ */
  gtk_signal_connect (GTK_OBJECT (ok_btn), "clicked",
                      GTK_SIGNAL_FUNC (gsearch_ok_cb),
                      (gpointer)gsearch_wnd);
  gtk_signal_connect (GTK_OBJECT (cancel_btn), "clicked",
                      GTK_SIGNAL_FUNC (gsearch_cancel_cb),
                      (gpointer)gsearch_wnd);

  /* ܥܥå˥ǡɤ߹ */
  init_combo_list();
  cm_dir.glist = g_list_first(cm_dir.glist);
  if (cm_dir.glist != NULL) {
    if ((cm_dir.glist)->data != NULL) {
      gtk_combo_set_popdown_strings(GTK_COMBO(cm_dir.combo), cm_dir.glist);
    }
  }
  cm_ftype.glist = g_list_first(cm_ftype.glist);
  if (cm_ftype.glist != NULL) {
    if ((cm_ftype.glist)->data != NULL) {
      gtk_combo_set_popdown_strings(GTK_COMBO(cm_ftype.combo), cm_ftype.glist);
    }
  }
  cm_pat.glist = g_list_first(cm_pat.glist);
  if (cm_pat.glist != NULL) {
    if ((cm_pat.glist)->data != NULL) {
      gtk_combo_set_popdown_strings(GTK_COMBO(cm_pat.combo), cm_pat.glist);
    }
  }

  /* ǥեȤΥեȤ */
  font_desc = pango_font_description_from_string(text_font);
  gtk_widget_modify_font(dummy_label, font_desc);
  gtk_widget_modify_font(dir_label, font_desc);
  gtk_widget_modify_font(sel_btn, font_desc);
  gtk_widget_modify_font(ftype_label, font_desc);
  gtk_widget_modify_font(pat_label, font_desc);
  gtk_widget_modify_font(ok_btn, font_desc);
  gtk_widget_modify_font(cancel_btn, font_desc);
  gtk_widget_modify_font(cm_dir.entry, font_desc);
  gtk_widget_modify_font(cm_ftype.entry, font_desc);
  gtk_widget_modify_font(cm_pat.entry, font_desc);
  pango_font_description_free(font_desc);

  return gsearch_wnd;
}

/**************
   顼
 **************/
void cme_modify_colors(struct text_struct *doc, GtkWidget *message_entry)
{
  GdkColor color;

  if (gdk_color_parse(cm_line_bg_color, &color) == FALSE) {
    color.red = 0xffff;
    color.green = 0xffff;
    color.blue = 0xffff;
  }
  gtk_widget_modify_bg(GTK_WIDGET(doc->textbox), GTK_STATE_NORMAL, &color);

  if (gdk_color_parse(cm_line_fg_color, &line_fg_color) == FALSE) {
    line_fg_color.red = 0x0000;
    line_fg_color.green = 0x0000;
    line_fg_color.blue = 0x0000;
  }
  gtk_widget_modify_fg(GTK_WIDGET(doc->textbox), GTK_STATE_NORMAL, &line_fg_color);

  if (gdk_color_parse(cm_ctrl_color, &ctrl_color) == FALSE) {
    ctrl_color.red =  0x8888;
    ctrl_color.green = 0x6666;
    ctrl_color.blue = 0x5555;
  }

  if (gdk_color_parse (cm_text_bg_color, &color) == FALSE) {
    color.red = 0xffff;
    color.green = 0xffff;
    color.blue = 0xffff;
  }
  gtk_widget_modify_base(GTK_WIDGET(doc->textbox), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_base(GTK_WIDGET(message_entry), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_base(GTK_WIDGET(cm_search.entry), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_base(GTK_WIDGET(cm_jump.entry), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg(GTK_WIDGET(message_entry), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg(GTK_WIDGET(cm_search.entry), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg(GTK_WIDGET(cm_jump.entry), GTK_STATE_NORMAL, &color);

  if (gdk_color_parse (cm_basic_color, &color) == FALSE) {
    color.red = 0x0000;
    color.green = 0x0000;
    color.blue = 0x0000;
  }
  gtk_widget_modify_fg(GTK_WIDGET(message_entry), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_fg(GTK_WIDGET(cm_search.entry), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_fg(GTK_WIDGET(cm_jump.entry), GTK_STATE_NORMAL, &color);

  gtk_widget_modify_text(GTK_WIDGET(doc->textbox), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_text(GTK_WIDGET(message_entry), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_text(GTK_WIDGET(cm_search.entry), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_text(GTK_WIDGET(cm_jump.entry), GTK_STATE_NORMAL, &color);

  return;
}

/**********************
   Ƽץѥƥ
 **********************/
void cme_set_properties(void)
{
  /* ġСɽ */
  if (cm_toolbar_visible != 0) {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(toolbar_visible_item), TRUE);
  }

  /* ޤ֤ */
  if (cm_line_wrap != 0) {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(line_wrap_item), TRUE);
  }

  /* ֹɽ */
  if (cm_line_number != 0) {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(line_numbers_item), TRUE);
  }

  /* ʸĴɽ */
  if (cm_syn_highlight != 0) {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(syntax_highlighting_item),
                                   TRUE);
  }

  return;
}

/******************
   ġСɽ
 ******************/
void set_cme_toolbar_visible(int which)
{
  if (which == 0) {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(toolbar_visible_item), FALSE);
  }
  else {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(toolbar_visible_item), TRUE);
  }
  return;
}

/******************
   ޤ֤
 ******************/
void set_cme_line_wrap(int which)
{
  if (which == 0) {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(line_wrap_item), FALSE);
  }
  else {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(line_wrap_item), TRUE);
  }
  return;
}

/******************
   ֹɽ
 ******************/
void set_cme_line_numbers(int which)
{
  if (which == 0) {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(line_numbers_item), FALSE);
  }
  else {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(line_numbers_item), TRUE);
  }
  return;
}

/********************
   ʸĴɽ
 ********************/
void set_cme_syntax_highlighting(int which)
{
  if (which == 0) {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(syntax_highlighting_item),
                                   FALSE);
  }
  else {
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(syntax_highlighting_item),
                                   TRUE);
  }
  return;
}
