/*
 * GTK+(2.0.x)ѤPC-UNIXѥĢΥ
 *
 * 󥿡ե뤿δؿ
 */

/*
 * Cmemo : Text Editor
 * text_struct.c - source file for text_struct functions
 *
 * Copyright (C) 2000, 2001, 2002 Seiji Tamura
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <pango/pango.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cme_def.h"
#include "cmemo.h"
#include "files.h"
#include "linenum.h"
#include "coloron.h"
#include "undo.h"
#include "ini_file.h"
#include "interface.h"
#include "kanji_conv.h"
#include "cme_misc.h"
#include "bm_search.h"
#include "auto_indent.h"
#include "keys.h"

#define line_wrap_width 6
#define line_wrap_height 9
static unsigned char line_wrap_bits[] = {
  0x1e, 0x3e, 0x30, 0x30, 0x39, 0x1f, 0x0f, 0x0f, 0x1f, };

#define tab_img_width 6
#define tab_img_height 9
//static unsigned char tab_img_bits[] = {
//  0x00, 0x00, 0x04, 0x08, 0x1f, 0x08, 0x04, 0x00, 0x00, };
static unsigned char tab_img_bits[] = {
  0x00, 0x00, 0x02, 0x04, 0x0f, 0x04, 0x02, 0x00, 0x00, };

/* global variables */
int cm_im_editting;

/* local declarations */
static gint text_expose_cb(GtkWidget *widget,
                           GdkEventExpose *event,
                           gpointer user_data);
static gint text_area_expose(GtkWidget *widget,
                             GdkEventExpose *event,
                             gpointer user_data);
static void get_wraps(GtkTextView *text_view,
                      gint first_y,
                      gint last_y,
                      GArray *buffer_coords,
                      GArray *line_heights,
                      GArray *wraps,
                      gint *countp);
static gint line_wraps_expose(GtkWidget *widget,
                              GdkEventExpose *event,
                              gpointer user_data);
static void get_lines(GtkTextView *text_view,
                      gint first_y,
                      gint last_y,
                      GArray *buffer_coords,
                      GArray *numbers,
                      GArray *iter_height,
                      gint *countp);
static gint line_numbers_expose(GtkWidget *widget,
                                GdkEventExpose *event,
                                gpointer user_data);
static void new_document(struct text_struct *doc);


static void
gtk_text_view_commit_handler (GtkIMContext  *context,
                              const gchar   *str,
                              GtkTextView   *text_view)
{
//  gtk_text_view_commit_text (text_view, str);
  /*g_print("commit\n");*/
  cm_im_editting = 0;
}

static void
gtk_text_view_preedit_changed_handler (GtkIMContext *context,
				       GtkTextView  *text_view)
{
//  gchar *str;
//  PangoAttrList *attrs;
//  gint cursor_pos;
  gchar *str;
  PangoAttrList *attrs;
  gint cursor_pos;

//  gtk_im_context_get_preedit_string (context, &str, &attrs, &cursor_pos);
//  gtk_text_layout_set_preedit_string (text_view->layout, str, attrs, cursor_pos);
//
//  pango_attr_list_unref (attrs);
//  g_free (str);

  gtk_im_context_get_preedit_string(text_view->im_context, &str, &attrs, &cursor_pos);

  if (strlen(str) != 0) {
    /*g_print("true %s\n", str);*/
    cm_im_editting = 1;
  }
  else {
    /*g_print("false\n");*/
    cm_im_editting = 2;
  }
  g_free (str);
  /*g_print("preedit_changed\n");*/
}

//static gboolean
//gtk_text_view_retrieve_surrounding_handler (GtkIMContext  *context,
//					    GtkTextView   *text_view)
//{
//  GtkTextIter start;
//  GtkTextIter end;
//  gint pos;
//  gchar *text;
//
//  gtk_text_buffer_get_iter_at_mark (text_view->buffer, &start,  
//				    gtk_text_buffer_get_insert (text_view->buffer));
//  end = start;
//
//  pos = gtk_text_iter_get_line_index (&start);
//  gtk_text_iter_set_line_offset (&start, 0);
//  gtk_text_iter_forward_to_line_end (&end);
//
//  text = gtk_text_iter_get_slice (&start, &end);
//  gtk_im_context_set_surrounding (context, text, -1, pos);
//  g_free (text);
//
//  return TRUE;
//  g_print("retrieve_surrounding\n");
//}

//static gboolean
//gtk_text_view_delete_surrounding_handler (GtkIMContext  *context,
//					  gint           offset,
//					  gint           n_chars,
//					  GtkTextView   *text_view)
//{
//  GtkTextIter start;
//  GtkTextIter end;
//
//  gtk_text_buffer_get_iter_at_mark (text_view->buffer, &start,  
//				    gtk_text_buffer_get_insert (text_view->buffer));
//  end = start;
//
//  gtk_text_iter_forward_chars (&start, offset);
//  gtk_text_iter_forward_chars (&end, offset + n_chars);
//
//  gtk_text_buffer_delete (text_view->buffer, &start, &end);
//
//  return TRUE;
//  g_print("delete_surrounding\n");
//}

void connect_textbox(struct text_struct *doc, int flag)
{
  doc->exp_evt_id = gtk_signal_connect(GTK_OBJECT(doc->textbox), "expose_event",
                                       GTK_SIGNAL_FUNC(text_expose_cb), doc);
  return;
}

void disconnect_textbox(struct text_struct *doc, int flag)
{
  g_signal_handler_disconnect(G_OBJECT(doc->textbox), doc->exp_evt_id);
  return;
}

void connect_document(GtkTextBuffer *buffer, struct text_struct *doc)
{
  doc->ins_txt_id = g_signal_connect(G_OBJECT(buffer), "insert_text",
                                     GTK_SIGNAL_FUNC(doc_insert_text_cb), doc);
  doc->del_txt_id = g_signal_connect(G_OBJECT(buffer), "delete_range",
                                     GTK_SIGNAL_FUNC(doc_delete_text_cb), doc);
  doc->line_ins_id = g_signal_connect(G_OBJECT(buffer), "insert_text",
                                      GTK_SIGNAL_FUNC(line_count_insert), doc);
  doc->line_del_id = g_signal_connect(G_OBJECT(buffer), "delete_range",
                                      GTK_SIGNAL_FUNC(line_count_delete), doc);
  return;
}

void disconnect_document(GtkTextBuffer *buffer, struct text_struct *doc)
{
  g_signal_handler_disconnect(G_OBJECT(buffer), doc->ins_txt_id);
  g_signal_handler_disconnect(G_OBJECT(buffer), doc->del_txt_id);
  g_signal_handler_disconnect(G_OBJECT(buffer), doc->line_ins_id);
  g_signal_handler_disconnect(G_OBJECT(buffer), doc->line_del_id);
  /* Υ٥Ȥ³򷫤֤ȥ٥ */
  /* 򤦤ޤʤʤ褦Ǥ...              */
  /*gtk_signal_disconnect(GTK_OBJECT(cm_window), doc->key_release_id);*/
  return;
}

/*
 * update_text_color
 *
 * 襤٥ȤǤιʸĴɽ
 *
 */
static void
update_text_color(GtkTextView *text_view,
                  GtkTextIter *start_iter,
                  GtkTextIter *end_iter,
                  GtkTextIter *topl_iter,
                  GtkTextIter *btml_iter)
{
  int in_comment;
  int bm_rtn_cmt_start;
  int bm_rtn_cmt_end;
  gchar *textbuf;
  GtkTextBuffer *buffer;

  buffer = gtk_text_view_get_buffer(text_view);

  in_comment = -1;
  textbuf = gtk_text_buffer_get_text(buffer, topl_iter, start_iter, TRUE);
  if (textbuf != NULL) {
    bm_rtn_cmt_start = bm_re_search(textbuf,
                                 strlen(textbuf),
                                 cm_ft_inf[cm_input_mode].cmt_start,
                                 strlen(cm_ft_inf[cm_input_mode].cmt_end),
                                 0);
    bm_rtn_cmt_end = bm_re_search(textbuf,
                               strlen(textbuf),
                               cm_ft_inf[cm_input_mode].cmt_end,
                               strlen(cm_ft_inf[cm_input_mode].cmt_end),
                               0);
    if (bm_rtn_cmt_start != -1) {
      if (bm_rtn_cmt_start > bm_rtn_cmt_end) {
        in_comment = 1;
      }
      else {
        in_comment = 0;
      }
    }
    g_free(textbuf);
  }

  if (in_comment == -1) {
    textbuf = gtk_text_buffer_get_text(buffer, start_iter, btml_iter, TRUE);
    if (textbuf != NULL) {
      bm_rtn_cmt_start = bm_search(textbuf,
                                   strlen(textbuf),
                                   cm_ft_inf[cm_input_mode].cmt_start,
                                   strlen(cm_ft_inf[cm_input_mode].cmt_end),
                                   0);
      bm_rtn_cmt_end = bm_search(textbuf,
                                 strlen(textbuf),
                                 cm_ft_inf[cm_input_mode].cmt_end,
                                 strlen(cm_ft_inf[cm_input_mode].cmt_end),
                                 0);
      if (bm_rtn_cmt_end != -1) {
        if (bm_rtn_cmt_start > bm_rtn_cmt_end) {
          in_comment = 1;
        }
        else if (bm_rtn_cmt_start == -1) {
          in_comment = 1;
        }
        else {
          in_comment = 0;
        }
      }
      else {
        in_comment = 0;
      }
      g_free(textbuf);
    }
  }

  textbuf = gtk_text_buffer_get_text(buffer, start_iter, end_iter, TRUE);
  if (textbuf != NULL) {
    in_comment = apply_color(start_iter,
                             end_iter,
                             buffer,
                             textbuf,
                             &cm_ft_inf[cm_input_mode],
                             in_comment);
    if (gtk_text_buffer_get_selection_bounds(buffer, NULL, NULL) != TRUE) {
      gtk_text_view_place_cursor_onscreen(text_view);
    }
    g_free(textbuf);
  }

  return;
}

/*
 * update_ctrl_color
 *
 * ʸ
 *
 */
static void
update_ctrl_color(GtkTextView *text_view,
                  GtkTextIter *cur_iter,
                  GtkTextIter *end_iter,
                  GdkGC *gc,
                  GdkPixmap *img)
{
  gint x, y;
  GdkRectangle location;
  GdkDrawable *target;

  target = gtk_text_view_get_window(text_view, GTK_TEXT_WINDOW_TEXT);
  while (!gtk_text_iter_equal(cur_iter, end_iter)) {
    if (gtk_text_iter_get_char(cur_iter) == '\t') {
      /* ƥ졼GtkTextBufferκɸ */
      gtk_text_view_get_iter_location(text_view,
                                      cur_iter,
                                      &location);

      /* GtkTextBufferκɸ򥦥ɥκɸѴ */
      gtk_text_view_buffer_to_window_coords(text_view,
                                            GTK_TEXT_WINDOW_TEXT,
                                            location.x,
                                            location.y,
                                            &x, &y);
      /* ֤ι⤵Ĵ */
      y = y + location.height - tab_img_height;

      gdk_gc_set_stipple(gc, img);
      gdk_gc_set_fill(gc, GDK_STIPPLED);

      gdk_gc_set_ts_origin(gc, x, y);
      gdk_draw_rectangle(target, gc, TRUE, x, y, tab_img_width, tab_img_height);

      gdk_gc_set_ts_origin (gc, 0, 0);
      gdk_gc_set_fill (gc, GDK_SOLID);
    }
    gtk_text_iter_forward_char(cur_iter);
  }
  return;
}

/*
 * text_area_expose
 *
 * ƥΰκ襤٥
 *
 */
static gint
text_area_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
  GtkTextView *text_view;
  GdkDrawable *target;
  GdkGC *gc;
  GdkPixmap *img;
  GtkTextIter topl_iter;
  GtkTextIter btml_iter;
  GtkTextIter start_iter;
  GtkTextIter cur_iter;
  GtkTextIter end_iter;
  GdkRectangle location;
  gint x;
  gint y;
  GtkTextWindowType type;

  text_view = GTK_TEXT_VIEW(widget);

  type = GTK_TEXT_WINDOW_TEXT;
  target = event->window;

  /* create graphic context */
  gc = gdk_gc_new(event->window);
  gdk_gc_set_rgb_fg_color(gc, &ctrl_color);

  /* create bitmap */
  img = gdk_bitmap_create_from_data(target,
                                    (gchar*)tab_img_bits,
                                    tab_img_width,
                                    tab_img_height); 

  /*
   * ƥ졼
   */
  /* ɥκɸGtkTextBufferκɸѴ賫ϰ֤itr */
  gtk_text_view_window_to_buffer_coords(text_view, type,
                                        event->area.x, event->area.y,
                                        &x, &y);
  /*gtk_text_view_get_iter_at_location(text_view, &start_iter, x, y);*/
  gtk_text_view_get_iter_at_location(text_view, &start_iter, 0, y);

  /* ɥκɸGtkTextBufferκɸѴ轪λ֤itr */
  gtk_text_view_window_to_buffer_coords(text_view, type,
                                        event->area.x + event->area.width,
                                        event->area.y + event->area.height,
                                        &x, &y);
  gtk_text_view_get_iter_at_location(text_view, &end_iter, x, y);

  /* Ƥϰ(ɥ)γϹԤȽλԤitr */
  gtk_text_view_get_visible_rect(text_view, &location);
  /*gtk_text_view_get_iter_at_location(text_view,
                                     &topl_iter,
                                     location.x,
                                     location.y);*/
  gtk_text_view_get_iter_at_location(text_view,
                                     &topl_iter,
                                     0,
                                     location.y);
  gtk_text_view_get_iter_at_location(text_view,
                                     &btml_iter,
                                     location.x + location.width,
                                     location.y + location.height);

  /* ʸĴɽԤ */
  update_text_color(text_view, &start_iter, &end_iter, &topl_iter, &btml_iter);

  /* ʸʤɤʸ褹 */
  cur_iter = start_iter;
  update_ctrl_color(text_view, &cur_iter, &end_iter, gc, img);

  g_object_unref(G_OBJECT(gc));
  gdk_pixmap_unref(img);

  return(TRUE);
}

/*
 * get_wraps
 *
 * ޤ֤ɬפʹԤ
 *
 */
static void
get_wraps(GtkTextView *text_view,
          gint first_y,
          gint last_y,
          GArray *buffer_coords,
          GArray *line_heights,
          GArray *wraps,
          gint *countp)
{
  GtkTextIter disp_iter;
  GtkTextIter line_iter;
  gint count;
  gint size;
  GdkRectangle location;

  g_array_set_size(buffer_coords, 0);
  g_array_set_size(line_heights, 0);
  g_array_set_size(wraps, 0);

  /* Get iter at first y */
  gtk_text_view_get_line_at_y(text_view, &disp_iter, first_y, NULL);

  /* For each iter, get its location and add it to the arrays.
   * Stop when we pass last_y
   */
  count = 0;
  size = 0;

  while (!gtk_text_iter_is_end(&disp_iter)) {
    gint iswrap;

    gtk_text_view_get_iter_location(text_view, &disp_iter, &location);
    if (location.y >= last_y) {
      break;
    }
    /*printf("x:%d y:%d h:%d w:%d\n",
             location.x,
             location.y,
             location.height,
             location.width);*/
    g_array_append_val(buffer_coords, location.y);
    g_array_append_val(line_heights, location.height);
    line_iter = disp_iter;
    gtk_text_view_forward_display_line_end(text_view, &disp_iter);
    if (gtk_text_iter_ends_line(&line_iter) != TRUE) {
      gtk_text_iter_forward_to_line_end(&line_iter);
    }
    if (gtk_text_iter_equal(&disp_iter, &line_iter)) {
      iswrap = 0;
    }
    else {
      iswrap = 1;
    }
    g_array_append_val(wraps, iswrap);

    ++count;

    gtk_text_iter_forward_char(&disp_iter);
  }

  *countp = count;
  return;
}

/*
 * line_wraps_expose
 *
 * ޤ֤ΰκ襤٥
 *
 */
static gint
line_wraps_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
  gint count;
  GArray *pixels;
  GArray *heights;
  GArray *wraps;
  gint first_y;
  gint last_y;
  gint i;
  GtkTextView *text_view;
  GtkTextWindowType type;
  GdkDrawable *target;
  GdkGC *gc;
  GdkPixmap *img;

  text_view = GTK_TEXT_VIEW(widget);

  type = GTK_TEXT_WINDOW_RIGHT;
  target = event->window;

  /* create graphic context */
  gc = gdk_gc_new(event->window);
  /*gdk_gc_set_foreground(gc, &line_fg_color);*/
  gdk_gc_set_rgb_fg_color(gc, &line_fg_color);

  /* create bitmap */
  img = gdk_bitmap_create_from_data(target,
                                    (gchar*)line_wrap_bits,
                                    line_wrap_width,
                                    line_wrap_height);

  first_y = event->area.y;
  last_y = first_y + event->area.height;

  gtk_text_view_window_to_buffer_coords(text_view, type, 0,
                                        first_y, NULL, &first_y);

  gtk_text_view_window_to_buffer_coords(text_view, type, 0,
                                        last_y, NULL, &last_y);

  pixels = g_array_new(FALSE, FALSE, sizeof(gint));
  heights = g_array_new(FALSE, FALSE, sizeof(gint));
  wraps = g_array_new(FALSE, FALSE, sizeof(gint));

  get_wraps(text_view, first_y, last_y, pixels, heights, wraps, &count);

  gdk_gc_set_stipple(gc, img);
  gdk_gc_set_fill(gc, GDK_STIPPLED);
  for (i = 0; i < count; i++) {
    gint pos;

    gtk_text_view_buffer_to_window_coords(text_view, type, 0,
                                          g_array_index(pixels, gint, i),
                                          NULL, &pos);
    pos = pos + g_array_index(heights, gint, i) - line_wrap_height;
    if (g_array_index(wraps, gint, i) == 1) {
      gdk_gc_set_ts_origin(gc, 1, pos);
      gdk_draw_rectangle(target, gc, TRUE, 1, pos,
                         line_wrap_width, line_wrap_height);
    }
  }
  gdk_gc_set_ts_origin (gc, 0, 0);
  gdk_gc_set_fill (gc, GDK_SOLID);

  g_array_free(pixels, TRUE);
  g_array_free(heights, TRUE);
  g_array_free(wraps, TRUE);

  g_object_unref(G_OBJECT(gc));
  gdk_pixmap_unref(img);

  return(TRUE);
}

/*
 * get_lines
 *
 * ΰιֹ
 *
 */
static void
get_lines(GtkTextView *text_view,
          gint first_y,
          gint last_y,
          GArray *buffer_coords,
          GArray *numbers,
          GArray *iter_height,
          gint *countp)
{
  GtkTextIter iter;
  gint count;
  gint size;  

  g_array_set_size(buffer_coords, 0);
  g_array_set_size(numbers, 0);
  g_array_set_size(iter_height, 0);

  /* Get iter at first y */
  gtk_text_view_get_line_at_y(text_view, &iter, first_y, NULL);

  /* For each iter, get its location and add it to the arrays.
   * Stop when we pass last_y
   */
  count = 0;
  size = 0;

  while (!gtk_text_iter_is_end(&iter)) {
    gint y, height;
    gint line_num;

    gtk_text_view_get_line_yrange(text_view, &iter, &y, &height);

    g_array_append_val(buffer_coords, y);
    line_num = gtk_text_iter_get_line(&iter);
    g_array_append_val(numbers, line_num);
    g_array_append_val(iter_height, height);

    ++count;

    if ((y + height) >= last_y) {
      break;
    }
    gtk_text_iter_forward_line(&iter);
  }

  *countp = count;
  return;
}

/*
 * line_numbers_expose
 *
 * ֹΰκ襤٥
 *
 */
static gint
line_numbers_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
  gint i;
  gint count;
  GArray *pixels;
  GArray *numbers;
  GArray *heights;
  gint first_y;
  gint last_y;
  PangoLayout *layout;
  GtkTextView *text_view;
  GtkTextWindowType type;
  GdkDrawable *target;
  gchar line_num[64]; /* 64ʾιԿ̵... */
  gint pango_width;
  gint pango_height;
  gint prev_width = 0;
  GdkGC *gc;


  text_view = GTK_TEXT_VIEW(widget);
  type = GTK_TEXT_WINDOW_LEFT;
  target = event->window;

  /* create graphic context */
  gc = gdk_gc_new(event->window);
  gdk_gc_set_rgb_fg_color(gc, &line_fg_color);

  first_y = event->area.y;
  last_y = first_y + event->area.height;

  gtk_text_view_window_to_buffer_coords(text_view, type, 0,
                                        first_y, NULL, &first_y);
  gtk_text_view_window_to_buffer_coords(text_view, type, 0,
                                        last_y, NULL, &last_y);

  pixels = g_array_new(FALSE, FALSE, sizeof(gint));
  numbers = g_array_new(FALSE, FALSE, sizeof(gint));
  heights = g_array_new(FALSE, FALSE, sizeof(gint));

  get_lines(text_view, first_y, last_y, pixels, numbers, heights, &count);

  layout = gtk_widget_create_pango_layout(widget, "");

  for (i = 0; i < count; i++) {
    gint pos;
    gint l_num;

    gtk_text_view_buffer_to_window_coords(text_view, type, 0,
                                          g_array_index(pixels, gint, i),
                                          NULL, &pos);

    l_num = g_array_index(numbers, gint, i) + 1;
    sprintf(line_num, "%4d", l_num);

    pango_layout_set_text(layout, line_num, -1);

    pango_layout_get_pixel_size(layout,
                                &pango_width,
                                &pango_height);

    if (prev_width != pango_width) {
      gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(text_view),
                                           GTK_TEXT_WINDOW_LEFT,
                                           pango_width + 5);
    }
    prev_width = pango_width;

    gtk_paint_layout(widget->style, target, GTK_WIDGET_STATE(widget),
                     FALSE, NULL, widget, NULL, 2, pos + 1, layout);

    gdk_draw_line(event->window, gc,
                  pango_width + 3, pos,
                  pango_width + 3, pos + g_array_index(heights, gint, i));
  }

  g_array_free(pixels, TRUE);
  g_array_free(numbers, TRUE);
  g_array_free(heights, TRUE);

  g_object_unref(G_OBJECT(layout));
  g_object_unref(G_OBJECT(gc));

  return(TRUE);
}

/*
 * text_expose_cb
 *
 *
 *
 */
static gint
text_expose_cb(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
  GdkWindow *left_win;
  GdkWindow *right_win;
  GdkWindow *text_win;
  GtkTextView *text_view;

//  disconnect_textbox(user_data, 0);

  text_view = GTK_TEXT_VIEW(widget);

  left_win = gtk_text_view_get_window(text_view, GTK_TEXT_WINDOW_LEFT);
  right_win = gtk_text_view_get_window(text_view, GTK_TEXT_WINDOW_RIGHT);
  text_win = gtk_text_view_get_window(text_view, GTK_TEXT_WINDOW_TEXT);

  if (event->window == left_win) {
    line_numbers_expose(widget, event, user_data);
  }
  else if (event->window == text_win) {
    if (cm_im_editting != 1) {
      text_area_expose(widget, event, user_data);
    }
  }
  else if (event->window == right_win) {
    if (cm_im_editting != 1) {
      line_wraps_expose(widget, event, user_data);
    }
  }

//  connect_textbox(user_data, 0);

  return(FALSE);
}

/*
 * new_document
 *
 * ƥΰؿ³
 *
 */
static void new_document(struct text_struct *doc)
{
  GtkTextBuffer *buffer;
//  int pango_width;
//  int pango_height;
//  PangoLayout *layout;
  GtkTextView *view;

  doc->textbox = gtk_text_view_new();

  gtk_text_view_set_editable(GTK_TEXT_VIEW(doc->textbox), TRUE);
  /* GTK_WRAP_NONE
     GTK_WRAP_CHAR
     GTK_WRAP_WORD */
  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(doc->textbox), GTK_WRAP_NONE);

  /*layout = gtk_widget_create_pango_layout(doc->textbox, "");
  pango_layout_set_text(layout, "0000", -1);
  pango_layout_get_pixel_size(layout,
                              &pango_width,
                              &pango_height);
  g_object_unref(G_OBJECT(layout));*/

  /* Draw line numbers in the side windows; we should really */
  /* be more scientific about what width we set them to.     */
  /*gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(doc->textbox),
                                       GTK_TEXT_WINDOW_LEFT, pango_width + 5);*/
  /*gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(doc->textbox),
                                       GTK_TEXT_WINDOW_RIGHT, 8);*/

  /* ʲδؿɬǽ³ʤФʤʤ() */
  view = GTK_TEXT_VIEW(doc->textbox);
  g_signal_connect (G_OBJECT (view->im_context), "preedit_changed",
                    G_CALLBACK (gtk_text_view_preedit_changed_handler), view);
  g_signal_connect (G_OBJECT (view->im_context), "commit",
                    G_CALLBACK (gtk_text_view_commit_handler), view);
//  g_signal_connect (G_OBJECT (view->im_context), "retrieve_surrounding",
//                    G_CALLBACK (gtk_text_view_retrieve_surrounding_handler), view);
//  g_signal_connect (G_OBJECT (view->im_context), "delete_surrounding",
//                    G_CALLBACK (gtk_text_view_delete_surrounding_handler), view);
  /*doc->exp_evt_id = gtk_signal_connect(GTK_OBJECT(doc->textbox),
                                       "expose_event",
                                       GTK_SIGNAL_FUNC(line_numbers_expose),
                                       doc);*/
  connect_textbox(doc, 0);

  doc->modified = FALSE;
  doc->filename = NULL;

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(doc->textbox));
  /* debug */
  /*printf("new_document:buffer=%p\n", buffer);*/
  create_tags(buffer, &cm_ft_inf[cm_input_mode]);
  connect_document(buffer, doc);

  doc->key_prs_id = g_signal_connect(GTK_OBJECT(doc->textbox),
                                     "key_press_event",
                                     GTK_SIGNAL_FUNC(key_press_cb), doc);

  doc->key_rls_id = g_signal_connect(GTK_OBJECT(doc->textbox),
                                     "key_release_event",
                                     GTK_SIGNAL_FUNC(key_release_cb), doc);

  /* Υ٥ȤǤ³򷫤֤ȥ٥Ȥ */
  /* ޤʤʤ褦ʤΤǷҤäѤʤˤƤ */
  gtk_signal_connect(GTK_OBJECT(doc->textbox), "key_press_event",
                     GTK_SIGNAL_FUNC(auto_indent_cb), NULL);

  gtk_signal_connect(GTK_OBJECT(doc->textbox), "key_release_event",
                     GTK_SIGNAL_FUNC(auto_indent_cb), doc);

  gtk_signal_connect_after(GTK_OBJECT(doc->textbox), "move_cursor",
                           GTK_SIGNAL_FUNC(line_number_cb), doc);

//  gtk_signal_connect(GTK_OBJECT(doc->textbox), "key_release_event",
//                     GTK_SIGNAL_FUNC(line_number_cb), doc);

  gtk_signal_connect(GTK_OBJECT(doc->textbox), "button_release_event",
                     GTK_SIGNAL_FUNC(line_number_btn_cb), doc);
  return;
}

void set_tab_width(struct text_struct *doc, int tab_width)
{
  PangoTabArray *tabs;
  int pango_width;
  int pango_height;
  PangoLayout *layout;

  if ((tab_width > 0) && (tab_width <= 32)) {
    layout = gtk_widget_create_pango_layout(doc->textbox, " ");
    pango_layout_get_pixel_size(layout, &pango_width, &pango_height);
    g_object_unref(G_OBJECT(layout));
    tabs = pango_tab_array_new(1, TRUE);
    pango_tab_array_set_tab(tabs, 0, PANGO_TAB_LEFT, pango_width * tab_width);
    gtk_text_view_set_tabs(GTK_TEXT_VIEW(doc->textbox), tabs);
    pango_tab_array_free(tabs);
  }
  else {
    /* nothing to do */
  }
}

/**********************************
   ƥȥåȤ
 **********************************/
GtkWidget *create_scrolled_text_widget(struct text_struct *doc)
{
  GtkWidget *scrolled_window;
  PangoFontDescription *font_desc;
  char text_font[1024];

  /* ե̾ܥ */
  sprintf(text_font, "%s %d", cm_text_font, cm_font_size);

  /* ƥȤ */
  new_document(doc);
  gtk_widget_show(doc->textbox);

  /* ǥեȤΥեȤ */
  font_desc = pango_font_description_from_string(text_font);
  gtk_widget_modify_font(doc->textbox, font_desc);
  pango_font_description_free(font_desc);

  /* (ºݤɽƤǤʤfont꤬ȿǤ */
  /* 餷ǤʤΤǤǤϽʤ)   */
//  set_tab_width(doc, cm_tab_width);

  /* ޡ */
  /*gtk_text_view_set_left_margin(GTK_TEXT_VIEW(doc->textbox), 2);*/

  /* ޡ */
  /*gtk_text_view_set_right_margin(GTK_TEXT_VIEW(doc->textbox), 2);*/

  /*ܥå롣*/
  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
                                 GTK_POLICY_ALWAYS,  /* h_scrollbar */
                                 GTK_POLICY_ALWAYS); /* v_scrollbar */
  gtk_widget_show(scrolled_window);
  /* GTK_SHADOW_NONE       */
  /* GTK_SHADOW_IN         */
  /* GTK_SHADOW_OUT        */
  /* GTK_SHADOW_ETCHED_IN  */
  /* GTK_SHADOW_ETCHED_OUT */
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window),
                                      GTK_SHADOW_IN);

  /* ƥȥåȤ򥹥ܥåξ֤ */
  gtk_container_add(GTK_CONTAINER(scrolled_window), doc->textbox);

  return(scrolled_window);
}
